package meta

import (
	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"
)

// Conf object is used to pass parameters to functions in Meta package
// and generate ObjectMeta struct.
type Mutate interface {
	// Instance is the Owner Object which manages the Object.
	GetInstance() interfaces.Object
	// Name defines the name for the Object. This is used to populate
	// the field of same name in ObjectMeta
	GetObject() interfaces.Object
	GetName() string
	// Namespace defines the namespace in which Object is/will be
	// present. This is used to populate the field of same name in
	// ObjectMeta
	GetNamespace() string
	// GenLabelsFunc is used to generate labels for Object. The
	// generated string map populates the Labels in ObjectMeta.
	GenLabelsFunc() (map[string]string, error)
	// GenAnnotationsFunc is used to generate annotations for the
	// Object. The generated string map populates the Annotations in
	// ObjectMeta
	GenAnnotationsFunc() (map[string]string, error)
	// GenFinalizersFunc is used to generate finalizers for the
	// Object. The generated string slice populates the Finalizers in
	// ObjectMeta
	GenFinalizersFunc() ([]string, error)
	// AppendLabels is used to determine if labels from Owner Object
	// are to be inherited.
	//	AppendLabels bool
	ObjectType() string
	ObjectOwnerType() string
}

type GenLabels func(interfaces.Object) (map[string]string, error)
type GenAnnotations func(interfaces.Object) (map[string]string, error)
type GenFinalizers func(interfaces.Object) ([]string, error)

type Conf struct {
	Instance       interfaces.Object
	Object         interfaces.Object
	Namespace      string
	Name           string
	OwnerReference bool
	GenFinalizers
	GenLabels
	GenAnnotations
}

type Object interface {
	MutateMeta()
	MutateSpec()
	OwnerRef() bool
}
