package pod

import (
	"fmt"

	"github.com/ankitrgadiya/operatorlib/pkg/container"
	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"
	"github.com/ankitrgadiya/operatorlib/pkg/meta"

	"github.com/pkg/errors"
	corev1 "k8s.io/api/core/v1"
)

// GenerateConfigMap generates ConfigMap object as per the `Conf` struct passed
func MutatePod(c Mutate) (pod corev1.Pod, err error) {
	//	var con corev1.Container
	var containers []corev1.Container
	//	var mutateContainers []container.Mutate
	var volumes []corev1.Volume
	var securityContext *corev1.PodSecurityContext
	var restartPolicy corev1.RestartPolicy

	om, err := meta.MutateObjectMeta(c)

	if err != nil {
		return pod, errors.Wrap(err, "failed to generate objectmeta")
	}

	mutateContainers := c.Containers()
	for _, con := range mutateContainers {
		cont, err := container.MutateContainer(con)
		if err != nil {
			return pod, errors.Wrap(err, "failed to generate pod template spec")
		}
		//			container = *cont
		containers = append(containers, *cont)
	}

	volumes, err = c.Volumes()
	if err != nil {
		return pod, errors.Wrap(err, "failed to generate volumes for pod template spec")
	}
	securityContext, err = c.SecurityContext()
	if err != nil {
		return pod, errors.Wrap(err, "failed to generate securityContext pod template spec")
	}

	restartPolicy, err = c.RestartPolicy()
	if err != nil {
		return pod, errors.Wrap(err, "failed to generate securityContext pod template spec")
	}

	// pod = c.GetObject()
	pod.ObjectMeta = *om

	pod.Spec = corev1.PodSpec{
		Containers:      containers,
		Volumes:         volumes,
		SecurityContext: securityContext,
		RestartPolicy:   restartPolicy,
	}

	return pod, err
}

func (c Conf) Containers() []container.Mutate {
	var spec []container.Mutate
	if c.GenContainers != nil {
		return c.GenContainers
	}
	return spec
}

func (c Conf) Volumes() ([]corev1.Volume, error) {
	var spec []corev1.Volume
	if c.GenVolumesFunc != nil {
		return c.GenVolumesFunc()
	}
	return spec, nil
}
func (c Conf) SecurityContext() (*corev1.PodSecurityContext, error) {
	var spec *corev1.PodSecurityContext
	if c.GenSecurityContextFunc != nil {
		return c.GenSecurityContextFunc()
	}
	return spec, nil
}
func (c Conf) RestartPolicy() (corev1.RestartPolicy, error) {
	var spec corev1.RestartPolicy
	if c.GenRestartPolicyFunc != nil {
		return c.GenRestartPolicyFunc()
	}
	return spec, nil
}

//func (c Conf) GetObject() interfaces.Object {
//	obj := &corev1.Pod{}
//	obj = c.Object
//	return obj
//&}

func (c Conf) GetObject() interfaces.Object {
	return c.Object
}

func (c Conf) GetInstance() interfaces.Object {
	return c.Meta.GetInstance()
}
func (c Conf) GetName() string {
	return c.Meta.GetName()
}
func (c Conf) GetNamespace() string {
	return c.Meta.GetNamespace()
}

func (c Conf) GenLabelsFunc() (map[string]string, error) {
	var labels map[string]string
	if c.Meta.GenLabelsFunc != nil {
		return c.Meta.GenLabelsFunc()
	}
	return labels, nil
}
func (c Conf) GenAnnotationsFunc() (map[string]string, error) {
	var annotations map[string]string
	if c.Meta.GenAnnotationsFunc != nil {
		return c.Meta.GenAnnotationsFunc()
	}
	return annotations, nil

}
func (c Conf) GenFinalizersFunc() ([]string, error) {
	if c.Meta.GenFinalizersFunc != nil {
		return c.Meta.GenFinalizersFunc()
	}
	return nil, nil

}

// ObjectType returns the type of the ObjectSyncer subject
func (s Conf) ObjectType() string {
	return fmt.Sprintf("%T", s.GetObject())
}

// OwnerType returns the type of the ObjectSyncer owner
func (s Conf) ObjectOwnerType() string {
	return fmt.Sprintf("%T", s.GetInstance())
}
