package pod

import (
	"github.com/ankitrgadiya/operatorlib/pkg/container"
	"github.com/ankitrgadiya/operatorlib/pkg/meta"

	corev1 "k8s.io/api/core/v1"
)

// GenConfigMapFunc defines a function which generates ConfigMap.
type GenPodFunc func(Conf) (*corev1.PodTemplateSpec, error)

// GenConfigMapFunc defines a function which generates ConfigMap.
type GenVolumesFunc func() ([]corev1.Volume, error)

type GenSecurityContextFunc func() (*corev1.PodSecurityContext, error)

// GenDataFunc defines a function which generates data (string map)
// for Configmap.
// type GenContainersFunc func() ([]container.Mutate, error)
type GenRestartPolicyFunc func() (corev1.RestartPolicy, error)

// Conf is used to pass parameters to functions in this package to
// perform operations on Configmap objects.
type Conf struct {
	Meta   meta.Mutate
	Object *corev1.Pod
	GenPodFunc
	GenContainers []container.Mutate
	//	InitContainers
	GenVolumesFunc
	GenSecurityContextFunc
	GenRestartPolicyFunc
}

type Mutate interface {
	meta.Mutate
	//Pod(Conf) (*corev1.PodTemplateSpec, error)
	Volumes() ([]corev1.Volume, error)
	SecurityContext() (*corev1.PodSecurityContext, error)
	RestartPolicy() (corev1.RestartPolicy, error)
	Containers() []container.Mutate
	//	GetObject() *corev1.Pod
}
