package pvc

import (
	"context"

	"github.com/ankitrgadiya/operatorlib/pkg/meta"

	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"

	"github.com/pkg/errors"
	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"

	"github.com/ankitrgadiya/operatorlib/syncer"
)

func (c Conf) Mutate() error {
	var om *metav1.ObjectMeta
	var accessModes []corev1.PersistentVolumeAccessMode
	var resources corev1.ResourceRequirements
	var err error

	om, err = meta.MutateObjectMeta(c.Meta)

	if err != nil {
		return errors.Wrap(err, "failed to generate objectmeta")
	}
	if c.GenAccessModesFunc != nil {
		accessModes, err = c.GenAccessModesFunc()
		if err != nil {
			return errors.Wrap(err, "failed to generate pvc accessmodes")
		}
	}
	if c.GenResourcesFunc != nil {
		resources, err = c.GenResourcesFunc()
		if err != nil {
			return errors.Wrap(err, "failed to generate pvc accessmodes")
		}
	}

	c.Object.ObjectMeta = *om
	c.Object.Spec = corev1.PersistentVolumeClaimSpec{
		AccessModes:      accessModes,
		Resources:        resources,
		StorageClassName: c.StorageClassName,
	}

	return nil
}

func (c Conf) MakeObject() *corev1.PersistentVolumeClaim {
	obj := &corev1.PersistentVolumeClaim{
		ObjectMeta: metav1.ObjectMeta{
			Name:      c.GetName(),
			Namespace: c.GetNamespace(),
		},
	}

	return obj
}

func SyncObject(c Conf, ctx context.Context) (syncer.SyncResult, error) {

	c.Object = c.MakeObject()

	PVCSyncer := syncer.NewObjectSyncer("App PVC", c.GetInstance(), c.Object, c.Reconcile, func() error {
		if err := c.Mutate(); err != nil {
			return err
		}
		return nil
	})

	result, err := syncer.Sync(ctx, PVCSyncer)
	if err != nil {
		return result, err
	}
	return result, nil
}

func (c Conf) GetInstance() interfaces.Object {
	return c.Meta.GetInstance()
}
func (c Conf) GetName() string {
	return c.Meta.GetName()
}
func (c Conf) GetNamespace() string {
	return c.Meta.GetNamespace()
}
